#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import abc
from typing import Optional, List, Tuple
import math
import time
import datetime as dt
from docx import Document
from docx.enum.style import WD_STYLE_TYPE
from PIL import ImageGrab, Image, ImageFile
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
import nardascripting.base.toolbox as tb


ImageFile.LOAD_TRUNCATED_IMAGES = True  # Prevents image truncation

from . import langhelper as lh
_ = lh.CURRENT_GETTEXT


class DocItem:
    """Class that represents a value to be documented."""

    def __init__(self, displ_name: str, value, unit: str = "", add_prefix: bool = False):
        """Initializes the DocItem class.

        :param displ_name: The display name of the value.
        :param value: The value to display.
        :param unit: The unit of the value.
        :param add_prefix: Adds a unit prefix if True.
        """
        self.displ_name = displ_name
        self.displ_value = tb.value_to_unit_string(value, unit, add_prefix)


class IDocDataItem(metaclass=abc.ABCMeta):
    """Interface class for View data classes"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        self._screenshot_base_path = screenshot_base_path
        self._view_index = view_index
        self._img_file = ""
        self._meas_settings: List[DocItem] = []
        self._meas_results: List[DocItem] = []

    @staticmethod
    @abc.abstractmethod
    def get_data_item_type() -> str:
        """Gets the view type represented by this class"""
        pass

    @staticmethod
    @abc.abstractmethod
    def get_displ_name() -> str:
        """Gets the display name of the view represented by this class"""
        pass

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return ""

    @property
    def img_file(self) -> str:
        """Gets the full screenshot filename of the view represented by this class."""
        return self._img_file

    @property
    def settings_headline(self) -> str:
        """Gets the settings headline for this data item. """
        return _("Settings")

    @property
    def meas_result_headline(self) -> str:
        """Gets the measurement result headline for this data item. """
        return _("Measurement Results")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 999

    @abc.abstractmethod
    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        pass

    def _take_screenshot(self, signalshark: SignalSharkDev) -> bool:
        """ Takes a screenshot:

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded
        """
        try:
            if self._screenshot_base_path is None or not self.screenshot_name:
                return False

            if self._view_index < 0:
                signalshark.scpi.view.size_restore()
            else:
                signalshark.scpi.view.set_select(self._view_index)
                time.sleep(1)
                signalshark.scpi.view.size_maximize()
            time.sleep(1.5)

            signalshark.disconnect()
            time.sleep(1)
            img_full_filename = str(self._screenshot_base_path.joinpath(self.screenshot_name + ".png"))
            screenshot = ImageGrab.grab()
            screenshot.save(img_full_filename, 'PNG')
            signalshark.connect()
            self._img_file = img_full_filename
            time.sleep(1)
            signalshark.scpi.view.size_restore()
            return True
        except:
            return False

    def _get_doc_style(self, document: Document, style_name: str, style_type: WD_STYLE_TYPE):
        """Gets a dedicated document style or its normal alternative.

        :param document: The Word document.
        :param style_name: The name of the desired style.
        :param style_type: The type of the desired style.
        :return: Dedicated document style
        """
        try:
            return document.styles[style_name]
        except:
            try:
                styles = document.styles
                type_styles = [s for s in styles if s.type == style_type]
                for stl in type_styles:
                    if "Normal" in stl.name:
                        return stl
                if len(type_styles) > 0:
                    return type_styles[0]
                return None
            except:
                return None

    def render_data_to_doc(self, document: Document) -> bool:
        """Renders measurement settings and data of this data item into the given document.

        :param document: The Word document.
        :return: Returns True if succeeded.
        """
        try:
            # Get styles
            stl_meas_setting = self._get_doc_style(document, "MeasSetting", WD_STYLE_TYPE.PARAGRAPH)
            stl_meas_result = self._get_doc_style(document, "MeasResult", WD_STYLE_TYPE.PARAGRAPH)

            # Render settings
            document.add_heading(self.settings_headline, 2)
            for setting in self._meas_settings:
                paragraph = document.add_paragraph()
                paragraph.style = stl_meas_setting
                paragraph.add_run("{}: ".format(setting.displ_name)).bold = True
                paragraph.add_run(setting.displ_value)

            # Render measurement results
            if len(self._meas_results) < 1:
                return True
            document.add_heading(self.meas_result_headline, 2)
            for result in self._meas_results:
                paragraph = document.add_paragraph()
                paragraph.style = stl_meas_result
                paragraph.add_run("{}: ".format(result.displ_name)).bold = True
                paragraph.add_run(result.displ_value)

            return True
        except Exception as e:
            return False


@dataclass
class DocData:
    """Class that encapsulates the data to be documented."""
    meas_name: str = field(default="", metadata={"XmlName": "MeasName"})
    time_stamp: str = field(default=dt.datetime.now(tz.tzlocal()),
                               metadata={"XmlName": "TimeStamp", "Format": "%Y-%m-%dT%H:%M%z"})
    author_name: str = field(default="", metadata={"XmlName": "AuthorName"})
    comment: str = field(default="", metadata={"XmlName": "Commen"})
    data_items: List[IDocDataItem] = field(default_factory=lambda: [], metadata={"XmlName": "DocDataItems"})

