#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import wx
import datetime as dt
from .mdocdlgsbase import *
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import *
from .. import filehandling as fh
from .. import langhelper as lh
_ = lh.CURRENT_GETTEXT


class MDocSettingsDlg(MDocSettingsDlgBase):
    """Custom dialog class for general mdoc settings."""

    def __init__(self, parent):
        """Dialog class for data visualization.

        :param parent: The parent window as wx.Window.
        """
        super().__init__(parent=parent)
        # load general settings:
        self.gen_settings = fh.load_from_json(fh.GEN_SETTINGS_FILE_NAME)

        default_file = fh.get_file_path(fh.DOC_TEMPL_FILE_NAME)
        templ_file = self.gen_settings.get("fpck_templ_file", str(default_file))
        self.fpck_templ_file.SetPath(templ_file)

        self.tctr_default_meas_name.SetValue(self.gen_settings.get("tctr_default_neas_name", "Meas_"))

        lang_list = lh.get_supported_languages()
        language = lh.CURRENT_LANGUAGE
        self.cbx_language.SetItems(lang_list)
        if language in lang_list:
            self.cbx_language.SetStringSelection(language)


    def on_ok_button_click(self, event):
        """Saves the settings to file"""
        global _
        self.gen_settings["fpck_templ_file"] = self.fpck_templ_file.GetPath()
        self.gen_settings["tctr_default_meas_name"] = self.tctr_default_meas_name.GetValue()
        sel_lang = self.cbx_language.GetValue()
        lh.change_language(sel_lang)
        fh.save_as_json(self.gen_settings, fh.GEN_SETTINGS_FILE_NAME)
        event.Skip()


class DocSettingsDlg(DocSettingsDlgBase):
    def __init__(self, parent, task_list: list, task_name: str, default_meas_dir_name: str):
        super().__init__(parent)

        # Get measurement tasks
        self.tasklist = task_list
        self.task_name = task_name
        for item in self.tasklist:
            self.cbx_task_name.AppendItems(str(item))
        self.cbx_task_name.SetStringSelection(self.task_name)

        # Load last doc settings
        self.doc_settings = fh.load_from_json(fh.LAST_DOC_SETTINGS_FILE_NAME)
        base_working_dir = self.doc_settings.get("base_working_dir", "D:\\Narda_SignalShark\\Datalogger")
        self.dirpick_base_working_dir.SetPath(base_working_dir)

        timestamp_dt = dt.datetime.now()
        timestamp_str = timestamp_dt.strftime("%Y-%m-%d_%H-%M")
        self.posix_timestamp = timestamp_dt.timestamp()
        new_folder_name = "{}_{}".format(default_meas_dir_name, timestamp_str)
        self.tctr_meas_name.SetValue(new_folder_name)
        self.tctr_author_name.SetValue(self.doc_settings.get("author_name", ""))

        author_name = self.doc_settings.get("author_name", "")
        self.tctr_author_name.SetValue(author_name)

    # Virtual event handlers, overide them in your derived class
    def on_ok_button_click( self, event ):
        self.doc_settings["base_working_dir"] = self.dirpick_base_working_dir.GetPath()
        self.doc_settings["meas_name"] = self.tctr_meas_name.GetValue()
        self.doc_settings["selected_task"] = self.cbx_task_name.GetStringSelection()
        self.doc_settings["author_name"] = self.tctr_author_name.GetValue()
        self.doc_settings["comment"] = self.tctr_comment.GetValue()
        self.doc_settings["posix_timestamp"] = self.posix_timestamp
        fh.save_as_json(self.doc_settings, fh.LAST_DOC_SETTINGS_FILE_NAME)
        event.Skip()


# noinspection PySimplifyBooleanCheck
class MDocProgress(ProgressDlg):
    def __init__(self, parent, message: str = "", caption: str = "ProgressDlg",
                 btn_style=wx.CANCEL, icon_style=None, pos=wx.DefaultPosition):
        """Dialog initialization"""
        super().__init__(parent, message, caption, btn_style, icon_style, pos)
        self._main_frm = parent
        self.SetTitle(_("Measurement Report Progress"))


    def update_progress(self, evt: ProgressEvent):
        """Receives data from thread and updates progressbar and message text

        :param evt: Event of type ProgressEvent.
        """

        # Handle data:
        if evt.data is not None:
            if evt.data == False:
                self.Iconize(True)
            else:
                self.Iconize(False)

        # Update dialog if visible:
        if self.IsShown():
            super().update_progress(evt)